/*
 *  GNOM - packet radio-terminal for Linux
 *  Copyright (C) 1997 Jochen Sarrazin, DG6VJ. All rights reserved.
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/****************************************************************************/
/**  Routinen, um die ganzen Puffer auf der Platte zwischenzuspeichern,    **/
/**  und sie beim nchsten Start wieder zu holen                           **/
/****************************************************************************/

#include "global.h"



#define CHANSTAT_VERSION 1		/* Version 1 des Files */




/*
 * Speichert die Rufzeichen ab, ber die wir connected haben. Wichtig, da nur von diesen Calls
 * reconnected - Meldungen erlaubt werden.
 */
static void save_conncalls(int fd, int chan)
{
  s_conncalls *tmp;
  int count=0;
  
  tmp = chaninfo[chan]->conncalls;
  
  while (tmp != NULL)
  {
    count++;
    tmp = tmp->next;
  }
  
  write(fd,&count,sizeof(count));
  
  tmp = chaninfo[chan]->conncalls;
  
  while (tmp != NULL)
  {
    write(fd,tmp->call,sizeof(tmp->call));
    tmp = tmp->next;
  }
}



/*
 * Speichert die Rufzeichen, via die wir connected haben, in der conn - struktur ab.
 */
static void restore_conncalls(int fd, s_conncalls *conn)
{
  int count;
  s_conncalls *tmp;
  
  if (read(fd,&count,sizeof(count))<0) return;
  if (count == 0) return;
  while (count > 0)
  {
  
    if (conn == NULL)
    {
      conn = (s_conncalls *)malloc(sizeof(s_conncalls));
      tmp = conn;
    }
    else
    {
      tmp = conn;
      while (tmp->next != NULL) tmp = tmp->next;
    
      tmp->next = (s_conncalls *)malloc(sizeof(s_conncalls));
      tmp = tmp->next;
    }
    
    tmp->next = NULL;
    read(fd,tmp->call,sizeof(tmp->call));
    
    count--;
  }
}



/*
 * Speichert den Status aller Kanle in ein File
 */
int save_channelstatus()
{
  int fd,i,k;
  char tmp[500];
  
  
  sprintf(tmp,"%s/chanstat.gnom",config->maindir);
  if ((fd = open(tmp,O_CREAT|O_TRUNC|O_WRONLY)) == -1) return(FALSE);

  /* Zugriffsrechte einstellen */
  fchmod(fd,S_IRUSR|S_IWUSR);
  
  /* Versions-Info schreiben */
  tmp[0] = CHANSTAT_VERSION;
  write(fd,&tmp[0],1);
  
  /* Jetzt kommt fr jeden Kanal ein Block mit Informationen - natrlich nur connectete Kanle. */
  i = 1;
  while (i < chancount)
  {
    if (chaninfo[i]->driverfd != -1)
    {
      /* Felder aus dem chaninfo[] - Array, die bentigt werden */    
      write(fd,chaninfo[i]->mycall,sizeof(chaninfo[i]->mycall));
      write(fd,chaninfo[i]->call,sizeof(chaninfo[i]->call));
      write(fd,chaninfo[i]->firstcall,sizeof(chaninfo[i]->firstcall));

      if (chaninfo[i]->digis == NULL)
         k = 0;
      else
         k = strlen(chaninfo[i]->digis)+1;
      write(fd,&k,sizeof(k));
      if (k > 0)
      write(fd,chaninfo[i]->digis,k);

      if (chaninfo[i]->name == NULL)
         k = 0;
      else
         k = strlen(chaninfo[i]->name)+1;
      write(fd,&k,sizeof(k));
      if (k > 0)
      write(fd,chaninfo[i]->name,k);
      
      write(fd,&chaninfo[i]->flags,sizeof(chaninfo[i]->flags));
      write(fd,&chaninfo[i]->oldflags,sizeof(chaninfo[i]->oldflags));
      write(fd,&chaninfo[i]->type,sizeof(chaninfo[i]->type));
      write(fd,&chaninfo[i]->logintime,sizeof(chaninfo[i]->logintime));
      write(fd,&chaninfo[i]->status,sizeof(chaninfo[i]->status));
      write(fd,&chaninfo[i]->oldstatus,sizeof(chaninfo[i]->oldstatus));
      write(fd,&chaninfo[i]->consstat,sizeof(chaninfo[i]->consstat));
      write(fd,&chaninfo[i]->convert_tx,sizeof(chaninfo[i]->convert_tx));
      write(fd,&chaninfo[i]->convert_rx,sizeof(chaninfo[i]->convert_rx));
      
      save_conncalls(fd,i);
      
      write(fd,&chaninfo[i]->rx_bytes,sizeof(chaninfo[i]->rx_bytes));
      write(fd,&chaninfo[i]->tx_bytes,sizeof(chaninfo[i]->tx_bytes));
      
      write(fd,&chaninfo[i]->qsonr,sizeof(chaninfo[i]->qsonr));
    }
    
    i++;
  }


  close(fd);

  return(TRUE);  
}



void restore_channelstatus()
{
  int fd,k,i,found;
  char tmp[500];
  
  char mycall[20];
  char call[20];
  char firstcall[20];
  char *digis=NULL;
  char *name=NULL;
  uint flags;
  uint oldflags;
  uchar type;
  time_t logintime;
  uchar status;
  uchar oldstatus;
  uchar consstat;
  int convert_rx;
  int convert_tx;
  int qsonr;
  s_conncalls *conn;
  unsigned long rx_bytes;
  unsigned long tx_bytes;
  
  
  
  sprintf(tmp,"%s/chanstat.gnom",config->maindir);
  if ((fd = open(tmp,O_RDONLY)) == -1) return;
  
  read(fd,&tmp[0],1);
  
  if (tmp[0] != CHANSTAT_VERSION) return;
  
  read(fd,mycall,sizeof(mycall));
  read(fd,call,sizeof(call));
  read(fd,firstcall,sizeof(firstcall));

  read(fd,&k,sizeof(k));
  if (k != 0)
  {
    digis = (char *)malloc(k);
    read(fd,digis,k);
  }

  read(fd,&k,sizeof(k));
  if (k != 0)
  {
    name = (char *)malloc(k);
    read(fd,name,k);
  }
  
  read(fd,&flags,sizeof(flags));
  read(fd,&oldflags,sizeof(oldflags));
  read(fd,&type,sizeof(type));
  read(fd,&logintime,sizeof(logintime));
  read(fd,&status,sizeof(status));
  read(fd,&oldstatus,sizeof(oldstatus));
  read(fd,&consstat,sizeof(consstat));
  read(fd,&convert_rx,sizeof(convert_rx));
  read(fd,&convert_tx,sizeof(convert_tx));
  
  conn = NULL;
  restore_conncalls(fd,conn);
  
  read(fd,&rx_bytes,sizeof(rx_bytes));
  read(fd,&rx_bytes,sizeof(tx_bytes));
  read(fd,&qsonr,sizeof(qsonr));


  /* Gucken, ob irgendwo ein Kanal ist, der das selbe firstcall und die selbe qsonr hat */
  i = 1;
  found = FALSE;
  while ((i <= chancount) && !found)
  {
    if ((chaninfo[i]->qsonr == qsonr) && (!strcmp(chaninfo[i]->firstcall,firstcall)))
    {
      found = TRUE;
      strcpy(chaninfo[i]->mycall,mycall);
      strcpy(chaninfo[i]->call,call);

      if (chaninfo[i]->digis != NULL) free(chaninfo[i]->digis);
      chaninfo[i]->digis = digis;
      if (chaninfo[i]->name != NULL) free(chaninfo[i]->name);
      chaninfo[i]->name = name;

      chaninfo[i]->flags = flags;
      chaninfo[i]->oldflags = oldflags;
      chaninfo[i]->type = type;
      chaninfo[i]->logintime = logintime;
      chaninfo[i]->status = status;
      chaninfo[i]->oldstatus = oldstatus;
      chaninfo[i]->consstat = consstat;
      chaninfo[i]->convert_rx = convert_rx;
      chaninfo[i]->convert_tx = convert_tx;
      chaninfo[i]->conncalls = conn;
      chaninfo[i]->rx_bytes = rx_bytes;
      chaninfo[i]->tx_bytes = tx_bytes;
  
    }
    i++;
  }


  if (digis != NULL)
  {
    free(digis);
    digis = NULL;
  }
  if (name != NULL)
  {
    free(name);
  }
  
  close(fd);
}



