/*
 *  GNOM - packet radio-terminal for Linux
 *  Copyright (C) 1997 Jochen Sarrazin, DG6VJ. All rights reserved.
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/****************************************************************************/
/**  Funktionen zur Ansteuerung des Lautsprechers im Hintergrund, also     **/
/**  ohne das aktuell laufende Programm zu stren.                         **/
/****************************************************************************/

#include "global.h"
#include "speaker.h"

#include "toolbox.h"

#include <string.h>


char *savestr;
pid_t pid;


/* Eine bestimmte Anzahl Microsekunden warten */
void sleep_usec(int us)
{
  struct timeval tv;		/* fr die zeit beim select() */

  tv.tv_sec = 0;
  tv.tv_usec = us;
  select(0,NULL,NULL,NULL,&tv);
}



/*
 * speaker_playstring
 * Diese Funktion spielt den bergebenen String im Lautsprecher ab.
 *
 * Format: ton,laenge;ton,laenge;ton,laenge....
 *
 * Die Lnge in Milli-Sekunden
 */
void speaker_playstring(char *str)
{
  int ton,laenge,i;
  char tmp2[100],tmp3[100];



  if (fork() != 0) return;		/* Der Elternprozess darf schnell zurckkehren */
  
  /* Den bergebenen String sichern */
  savestr = (char *)strdup(str);
  
  while (savestr[0] != '\0')
  {
    if ((i = POS(';',savestr)) != -1)
    {
      COPY(tmp2,savestr,0,i);
      COPY(savestr,savestr,i+1,strlen(savestr)-i-1);
    }
    else
    {
      strcpy(tmp2,savestr);
      savestr[0] = '\0';
    }
    
    /* Sooo... Jetzt wird die Lnge und die Tonhhe auseinanderklamsert */
    if ((i = POS(',',tmp2)) != -1)	/* Nur wenn auch ein Komma vorkommt, wird der String gespielt. */
    {
      COPY(tmp3,tmp2,0,i);
      COPY(tmp2,tmp2,i+1,strlen(tmp2)-i-1);
      
      ton = atoi(tmp3);
      laenge = atoi(tmp2);
      
      if (laenge != 0)		/* Lnge ok? */
      {
        ioctl(0,KIOCSOUND,ton);
        sleep_usec(laenge*1000);
        ioctl(0,KIOCSOUND,0);
      }
    }
  }
  
  free(savestr);
  exit(3);
}



/*
 * Klackt einmal im Lautsprecher
 */
void speaker_klack()
{
  ioctl(0,KIOCSOUND,50);
  sleep_usec(1);
  ioctl(0,KIOCSOUND,0);
}


